//  JAWTOpenGL.java -- JAWTOpenGL
//
//  04/14/06 Oleg Kobchenko - Created.
//  Copyright 2006 Oleg Kobchenko. All rights reserved.
//

package olegyk.jawtopengl.demo;

import java.awt.*;
import java.awt.event.*;
import javax.swing.Timer;
import javax.swing.*;
import java.util.*;
import olegyk.jawtopengl.*;

public class JAWTOpenGL extends JFrame
{
	static int offset = 0;
	Container pane = getContentPane();
	
    static {
        System.loadLibrary("OpenGLCanvas");
    }
    public JAWTOpenGL() {
        super("JAWTOpenGL");
		// removes frame and children, exits after last window closed
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        
		offset = (offset + 30) % 600;
		setLocation(offset, offset);
        setSize(500, 500);
    }
    
    public void addNotify() {
        super.addNotify();

        final OpenGLCanvas canvas = new OpenGLCanvas() {
			int w = 200, h = 200, step = 0;
			public void reshape(int width, int height) {
				w = width; h = height;
			}
			public void paint() {
				demo(w, h, step);
				step = (step + 1) % 360;
			}
		};
		final Label fpsLabel = new Label("0 fps");
        pane.setLayout(new BorderLayout(2, 2));
		pane.add(fpsLabel, BorderLayout.NORTH);
		pane.add(Box.createHorizontalStrut(10), BorderLayout.WEST);
		pane.add(Box.createHorizontalStrut(10), BorderLayout.EAST);
        pane.add(canvas, BorderLayout.CENTER);
		pane.add(new Label("  Fast, lightweight, native OpenGL"), BorderLayout.SOUTH);
		
		final Timer timer = new Timer(20, new ActionListener() {
			static final int freq = 40;
			long count = 0, time = System.currentTimeMillis();
			
			public void actionPerformed(ActionEvent evt) {
				canvas.repaint();
				if (0 == ++count % freq) {
					long now = System.currentTimeMillis();
					fpsLabel.setText(freq*1000/(now-time) + " fps");
					time = now;
				}
			}
		});
		canvas.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (timer.isRunning()) timer.stop(); else timer.start();
			}
		});
		addWindowListener(new WindowAdapter() {
			// attempt to paint canvas before it became
			// first visible will cause "invalid drawable"
			// during [OpenGLContext setView:]
			public void windowOpened(WindowEvent e) {
				timer.start();
			}
			public void windowClosing(WindowEvent e) {
				if (timer.isRunning()) timer.stop();
			}
		});
		
		JPopupMenu.setDefaultLightWeightPopupEnabled(false);
		int ACCL = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
		JMenuBar menuBar = new JMenuBar();
		{
			JMenu menu = new JMenu("File");
			
			menu.add(new AbstractAction("New") {
				public void actionPerformed(ActionEvent e) {
					new JAWTOpenGL().show();
				}
			}).setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ACCL));

			menu.add(new AbstractAction("Close") {
				public void actionPerformed(ActionEvent e) {
					JAWTOpenGL.this.dispatchEvent(new WindowEvent(
								  JAWTOpenGL.this, WindowEvent.WINDOW_CLOSING));
				}
			}).setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, ACCL));
			
			menuBar.add(menu);
		}
		setJMenuBar(menuBar);
    }
	    
    public static void main(String args[]) {
		new JAWTOpenGL().show();
    }    

	public static native void demo(int width, int height, int step);
}
