//  OpenGLCanvas.java -- JAWTOpenGL
//
//  04/14/06 Oleg Kobchenko - Created.
//  Copyright 2006 Oleg Kobchenko. All rights reserved.
//

package olegyk.jawtopengl;

import java.awt.*;

public class OpenGLCanvas extends Canvas {
	
	// overrides
	public void init() { }
	public void reshape(int width, int height) { }
	public void paint() { }
	
	// API
	public void reset(boolean init) {
		releaseOpenGL();
		allocOpenGL();
		if (init) {
			status = PaintStatusInit;
		}
	}
	
	// Implementation
	static final int PaintStatusInit = 0;
	static final int PaintStatusRun = 1;

	static final int CanvasStateReady = 0;
	static final int CanvasStateResized = 1;

	long openGLContext = 0;
	int status = PaintStatusInit;
	int state = CanvasStateReady;
    static {
        System.loadLibrary("OpenGLCanvas");
    }
	
	// prevent Canvas from interfering with OpenGL
    public void update(Graphics g) {	
        paint(g);
    }
	
    public void paint(Graphics g) {
		if (beginOpenGL()) {
			if (status == PaintStatusInit) {
				init();	 // override
				status = PaintStatusRun;
			}
			if (state == CanvasStateResized) {
				updateOpenGL();
				Dimension size = getSize();
				reshape((int)size.getWidth(), (int)size.getHeight());  // override
				state = CanvasStateReady;
			}
			paint();  // override
			endOpenGL();
		}
    }
	
	public void setBounds(int x, int y, int w, int h){
		super.setBounds(x, y, w, h);
		state = CanvasStateResized;
	}
    
	public void addNotify() {
        super.addNotify();
        allocOpenGL();
	}
	
	public void removeNotify() {
        releaseOpenGL();
        super.removeNotify();
	}
	
	public native void allocOpenGL();
	public native boolean beginOpenGL();
	public native void updateOpenGL();
	public native void endOpenGL();
	public native void releaseOpenGL();
}
